#ifndef MY_DATETYPE_H
#define MY_DATETYPE_H

#define LESS	-1
#define EQUAL	0
#define GREATER	1

// This is file DateType.h
// Here we declare the interface of a class so that we can
//	store the information of dates in objects of the DateType class and
//	call methods provided by these objects to process date information
// The actually implementation of the methods are inside DateType.cpp

#include <iostream>
using namespace std;

class DateType
{
 public:	//The following members can be called anywhere

  DateType();
  //The first constructor method to initialize a DateType object

  DateType(int newMonth, int newDay, int newYear);
  //The second constructor method to initialize a DateType object

  int YearIs();       // a method to returns year
  int MonthIs();      // a method to returns month
  int DayIs();        // a method to returns day

  bool SetDate(int newMonth, int newDay, int newYear);
  //A method to set a new date for the DateType object.
  //Return false if the date is invalid.

  void ReadDate();
  //A method to read in three integers and set them as the
  //	as the day, month, year of the DateType object to the console cout


  bool IsValidDate (int Month, int Day, int Year);
  //Check the input date to make sure it is a valid date.


  void PrintDate();
  //A method to output the contents of the DateType object to the console cout

  int ComparedTo(DateType aDate);
  // Pre: the object itself and aDate should have been initialized.
  // Post:return value should be
  // 	LESS, if the object itself comes before aDate.
  //	EQUAL, if the object itself is the same as aDate.
  //	GREATER, if the object itself comes after aDate.

  int ComparedCentury(DateType aDate);
	// Pre: Self and aDate have been initialized.
	// Post: return
	//	LESS, if the object itSelf comes before the century of aDate.
	//	EQUAL, if the object itSelf is in the same century as aDate.
	//	GREATER, if the object itSelf comes after the century of aDate.

  void AdvanceDays(int NumDays);
	//Modify the date by advancing it "NumDays" days into the future.

  void BackDays(int NumDays);
	//Modify the date by pushing it back "NumDays" days into the past.


 private: //The following members can be direclt accessed in DateType.cpp
  int year;
  int month;
  int day;



  //**********************************************************************
  // The following are new public member functions just added into DateType
  //**********************************************************************

  public:

  void SetRandomDate();
	//set the contents to a valid random date

  bool operator> (const DateType & aDate);
	//Define the relationl operator >

  bool operator< (const DateType & aDate);
	//Define the relationl operator <

  bool operator== (const DateType & aDate);
	//Define the relationl operator ==

  bool operator!= (const DateType & aDate);
	//Define the relationl operator !=

  bool operator>= (const DateType & aDate);
	//Define the relationl operator >=

  bool operator<= (const DateType & aDate);
	//Define the relationl operator <=

 //************************************************************
 //The following two global I/O operators are declared as friends
 //************************************************************
  friend ostream & operator<< (ostream & output, const DateType & aDate);
  friend istream & operator>> (istream & input, DateType & aDate);

};

#endif


