#include "Tree.h"


Tree :: Tree()
{
   Root = NULL;
}


Tree :: ~Tree()
{
 DeleteCompleteTree(Root);
}


void Tree :: Insert(DateType Val)
{
  InsertTree(Root, Val);
}


void Tree :: InsertTree(TreeNode * & SubtreeRoot, DateType Val)
{
   if (SubtreeRoot==NULL)
   {
    SubtreeRoot= new TreeNode;
    SubtreeRoot->Left=NULL;
    SubtreeRoot->Right=NULL;
    SubtreeRoot->Value = Val;
    return;
   }
// handle duplicate
   if (Val == SubtreeRoot->Value)
     return;

   //search Left
   if (Val < SubtreeRoot->Value)
     InsertTree(SubtreeRoot->Left, Val);

   //search Right
   else
     InsertTree(SubtreeRoot->Right,Val);
}


void Tree::Display()
{
 cout << endl;
 DisplayTree(Root);
 cout << endl;
}


void Tree :: DisplayTree(TreeNode * SubtreeRoot)
{
 if (SubtreeRoot != NULL)
   {
    DisplayTree(SubtreeRoot->Left);
    cout<<SubtreeRoot->Value<<' ';
    DisplayTree(SubtreeRoot->Right);
   }
}


bool Tree:: Search(DateType Val)
{
   return SearchTree(Root, Val);
}


bool Tree:: SearchTree(TreeNode * SubtreeRoot, DateType Val)
{
   if (SubtreeRoot == NULL)
     return false;
   if (SubtreeRoot->Value == Val)
     return true;
   if (Val < SubtreeRoot->Value)
     return SearchTree(SubtreeRoot->Left, Val);
   return SearchTree(SubtreeRoot->Right, Val);
}


void Tree :: DeleteCompleteTree(TreeNode * SubtreeRoot)
{
 if (SubtreeRoot != NULL)
   {
    DeleteCompleteTree(SubtreeRoot->Left);
    DeleteCompleteTree(SubtreeRoot->Right);
    delete SubtreeRoot;
   }
}


//************************************************************
// The following are three global functions as additional Code
// 	used to support the deletion of a node from a tree
//		Not in the book
//***************************************************



bool Tree::GetPredecessor(TreeNode * SubtreeRoot, DateType& data)
// Find the predecessor of the root node in SubtreeRoot.
// If there is one, return true and store the predecessor data in data
//Otherwise, return false.
{
  if (SubtreeRoot == NULL || (SubtreeRoot->Left) == NULL)
	  return false;

 TreeNode * temp = SubtreeRoot->Left;
  while (temp->Right != NULL)
    temp = temp->Right;

  data = temp->Value;
  return true;
}


void Tree::Delete(TreeNode *& SubtreeRoot, DateType item)
// Deletes item from tree.
// Post:  item is not in tree rooted at SubtreeRoot.
{
  if (SubtreeRoot == NULL)
	return;

  if (item < SubtreeRoot->Value)
    Delete(SubtreeRoot->Left, item);         // Look in Left subtree.
  else if (item > SubtreeRoot->Value)
    Delete(SubtreeRoot->Right, item);        // Look in Right subtree.
  else
    DeleteNode(SubtreeRoot);                 // Node found; call DeleteNode.

}

void Tree::DeleteNode(TreeNode *& SubtreeRoot)
// Deletes the node pointed to by SubtreeRoot.
{

  if (SubtreeRoot == NULL)
	return;
  //cout << "To delete " << tree->Value << endl;

  if (SubtreeRoot->Left != NULL  && SubtreeRoot->Right != NULL)
  { 
	DateType data;
    GetPredecessor(SubtreeRoot, data);
    SubtreeRoot->Value = data;
    Delete(SubtreeRoot->Left, data);       // Delete predecessor node.
  }
  else
  {
	TreeNode * tempPtr;
	tempPtr = SubtreeRoot;

	if (SubtreeRoot->Left == NULL)
		SubtreeRoot = SubtreeRoot->Right;
	else if (SubtreeRoot->Right == NULL)
		SubtreeRoot = SubtreeRoot->Left;
	delete tempPtr;
  }
}

void Tree::Clear()
{
	DeleteCompleteTree(Root);

	Root=NULL;
}

int Tree::BranchHeight(TreeNode * ptrNode)
{
	if (ptrNode==NULL)
		return 0;
	else if (BranchHeight(ptrNode->Left) > BranchHeight(ptrNode->Right))
		return 1+BranchHeight(ptrNode->Left);
	else
		return 1+BranchHeight(ptrNode->Right);
}

int Tree::Height()
{
	return BranchHeight(Root);
}



//*****************************************************************
// The following is an additional member function of the Tree class.
// 	It directly or indirectly uses the three functions above to
//	delete an item from the tree.
//		Not in the book
//***************************************************


void Tree::DeleteItem(DateType item)
// Calls recursive function Delete to delete item from tree.
{
  Delete(Root, item);
}


